/* gjdochtml-clean-layout.css
   Copyright (C) 2004 Free Software Foundation, Inc.

This file is part of GNU Classpath.

GNU Classpath is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.
 
GNU Classpath is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU Classpath; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307 USA. */

/* --- GNU Clean Style: Layout --- */

body
{
        background: white;
        color: black;
}

body.content
{
        margin-left: 1.3em;        
        margin-right: 1.3em;        
        margin-top: 0.9em;        
        margin-bottom: 1.3em;        
}

div[class="class title outer"]
{
        margin: 0em;
        margin-bottom: 1em;
        border: 1px solid;
        padding: 10pt;
}

[class="class title-package"]
{
        margin: 0em;
        padding: 0em;
}

[class="class title-class"]
{
        margin: 0em;
        margin-top: 0.2em;
        padding: 0em;
}

[class="table header"],
[class="table sub header"]
{
        margin: 0em;
        padding: 0.25em;
}

[class="section header"]
{
        margin: 0em;
        padding: 0.25em;
        border: 1px solid;
}

.serialized.package.header
{
        margin: 0em;
        padding: 0.25em;
        border: 1px solid;
        margin-bottom: 1em;
}


.serialized.class.header
{
        margin: 0em;
        padding: 0.25em;
        padding-left: 3.25em;
        border: 1px solid;
        text-indent: -3em;
        font-weight: normal;
        font-size: 1.2em;
}

.serialized.section.header
{
        font-size: 1.15em;
}

div.serialized.section
{
        padding-left: 1em;
}

.summary td,
.summary .inherited
{
        padding: 0.25em;
}

table[border]
{
        border-spacing: 0em;
        border-collapse: collapse;
}

table[border] td
{
        border-spacing: 0em;
        border-collapse: collapse;
}

hr
{
        border-top: 1px solid;
        border-bottom: 0px;
        border-left: 0px;
        border-right: 0px;
}

.class.summary .left
{
        width: 10%;
}

.class.summary td.left
{
        text-align: right;
}

.class.summary .left .synopsis
{
        font-size: small;
}


[class="package summary left"]
{
        width: 20%;
}

ul.inheritance
{       
        margin-left: 1.8em;
        padding-left: 0em;
        marker-offset: 0em !important;
        list-style-type: none;
        list-style-image: none;
}

li.inheritance
{
        margin: 0em !important;
        padding: 0em !important;
        marker-offset: 0em !important;
}

ul.inheritance > li:before
{
        display: marker;
        width: 12px;
        margin: 0em !important;
        padding: 0em !important;
        marker-offset: 0em !important;
        content: url(inherit.png);
}

[class="inheritance 0"] > li:before
{
        display: none;
}

ul [class="inheritance 0"]
{
        margin-left: 12px !important;
        padding-left: 0em;
        list-style-type: none !important;
        list-style-image: none !important;
}

.inheritance-tree
{
        position: relative;
        left: -24px;
}

.synopsis.outer
{
        padding: 0em;
        margin-bottom: 0.3em;
}

.synopsis.outer > *
{
        margin: 0em;
        padding: 0em;
        line-height: 1.0em;
}

div.navbar.div
{
        padding: 0em;
        margin: 0em;
        position: relative;
        left: -0.2em;
}

.navbar.div,
.navbar.div tr,
.navbar.div td
{
        margin: 0em;
        padding: 0em;
        border-spacing: 0em;
        border-collapse: collapse;
}

table.navbar.div
{
        width: 100%;
}

table.navbar.div td
{
        line-height: 1.9em;
        text-align: top;
}

.navbar.item
{
        /* float: left; */
        border: 1px solid;
        /* width: 7.5em; */
        text-align: center;
        margin: 0.05em;
        padding-top: 0.2em;
        padding-bottom: 0.3em;
        padding-left: 0.5em;
        padding-right: 0.5em;
        font-size: 0.75em;
        margin-bottom: 0.5em;
}

.navbar.item.active
{
        /* font-weight: bold; */
        border: 1px solid;
}

[class~="title"]
{
        border: 1px solid;
        padding: 0.3em;
        margin-top: 0em;
}

.table.container
{
        padding: 0em;
        margin: 0em;
}

.overview.summary
{
        padding: 0em;
        margin: 0em;
}

.summary
{
        margin-bottom: 1.2em;
}

table.navbar.top
{
        margin-bottom: 1em !important;
}

.package.description
{
        margin-top: 0.0em;
        margin-bottom: 1.0em;
}

[class="overview description top"]
{
        margin-top: 0.0em;
        margin-bottom: 1.0em;
}

[class="overview description full"]
{
        margin-top: 1.0em;
        margin-bottom: 0.0em;
}

.class.summary .right .list
{
        margin: 0em;
        padding: 0em;  
}

.class.summary .right .list .synopsis
{
        margin: 0em;
        padding: 0em;         
}

.classes .menu-subtitle
{
        padding: 0em;
        margin-top: 0.5em;
        margin-bottom: 0em;
}

.menu-title
{
        padding: 0em;
        margin-top: 0.8em;
        margin-bottom: 0.2em;
}

.menu-entry
{
        font-size: 0.8em;
}

.class.summary .right
{
        vertical-align: top;
}

.class.summary .right .list
{
        vertical-align: top;
}

.navi
{
        padding-top: 0.5em;
        font-size: x-small;
}

.navbar .header
{
        vertical-align: top;
}

.navbar .header span
{
        border: none;
        white-space: nowrap;
        padding-top: 0.2em;
        padding-bottom: 0.3em;
        padding-left: 0.5em;
}

.navbar .header a
{
        font-weight: bold;
        font-size: 11pt;
}

h1
{
        font-size: 1.5em;
}

h2
{
        font-size: 1.3em;
}

h3
{
        font-size: 1.1em;
}

.usage.summary .right .synopsis
{
        padding-left: 5.25em;
        text-indent: -5em;
}

.usage.summary .right .description
{
        padding-left: 3em;
        padding-top: 0.1em;
}

td.usage.summary
{
        vertical-align: top;
}

.index .entry .description
{
        padding-left: 3em;
}

.index .entry .key
{
        padding-left: 5.25em;
        text-indent: -5em;
}

.dep-list .summary .description
{
        padding-top: 0.1em;
        padding-left: 3em;
}

.dep-toc .header
{
        padding-bottom: 0em;
        margin-bottom: 0.2em;
}

.dep-toc .list
{
        margin-top: 0em;
        margin-bottom: 1.5em;
        padding-top: 0em;
        padding-bottom: 0em;
        padding-left: 2em;
}

ul.tree li
{
        list-style-type: disc;
}

ul.tree li div
{
        text-indent: -3em;
        padding-left: 3em;
}

ul.tree ul
{
        margin-left: 0em;
        padding-left: 1.8em;
}
